/* YaRock Player
* Copyright (C) 2010-2013 Sebastien amardeilh (sebastien.amardeilh+yarock@gmail.com)
*
* This program is free software: you can redistribute it and/or modify
* it under the terms of the GNU General Public License as published by
* the Free Software Foundation, either version 3 of the License, or
* (at your option) any later version.
*
* This program is distributed in the hope that it will be useful,
* but WITHOUT ANY WARRANTY; without even the implied warranty of
* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
* GNU General Public License for more details.
*
* You should have received a copy of the GNU General Public License
* along with this program.  If not, see <http://www.gnu.org/licenses/>.
*/
#include "CentralWidget/centraltoolbar.h"
#include "CentralWidget/menumodel.h"
#include "CentralWidget/navigatorbar.h"

#include "Widget/lineeditclear.h"
#include "Widget/spacer.h"

#include "Playlist/playlistwidget.h"

#include "Database/databasemanager.h"  // 0.0.53
#include "yarocksettings.h"            // 0.0.53
#include "threadmanager.h"             // 0.0.53
#include "global_actions.h"
#include "debug.h"

#include <QtGui>


CentralToolBar* CentralToolBar::INSTANCE = 0;
/*
********************************************************************************
*                                                                              *
*    Class MainToolButton                                                      *
*                                                                              *
********************************************************************************
*/
MainToolButton::MainToolButton( QWidget* parent ) : QToolButton(parent)
{
    this->setAutoRaise(true);
    this->setIcon(QIcon(":/images/settings.png"));
    this->setToolTip(tr("Tools"));
    this->setPopupMode(QToolButton::InstantPopup);   
      
    m_menuChooseDbAction = 0;
    m_menuChooseDb = new QMenu(tr("Choose database"));

    QMap<ENUM_ACTION, QAction*> *actions = ACTIONS();

    m_menu = new QMenu();

    QMenu *m0 = m_menu->addMenu(tr("&Show/Hide panel"));
    m0->addAction(actions->value(APP_SHOW_PLAYQUEUE));
    m0->addAction(actions->value(APP_SHOW_MENU));
    m0->addAction(actions->value(APP_SHOW_NOW_PLAYING));
    m_menu->addSeparator();
 
    QMenu *m1 = m_menu->addMenu(tr("&Add to playqueue"));
    m1->addAction(actions->value(PLAYQUEUE_ADD_FILE));
    m1->addAction(actions->value(PLAYQUEUE_ADD_DIR));
    m1->addAction(actions->value(PLAYQUEUE_ADD_URL)); 
    m_menu->addAction(actions->value(PLAYQUEUE_CLEAR));
    m_menu->addAction(actions->value(PLAYQUEUE_SAVE));
    m_menu->addAction(actions->value(PLAYQUEUE_AUTOSAVE));
    
    QMenu *m2 = m_menu->addMenu(tr("&Choose playlist mode"));
     
    QSignalMapper* mapper = new QSignalMapper();
    QActionGroup* group = new QActionGroup(this);
    addMenuItem(mapper, group, actions->value(PLAYQUEUE_MODE_TITLE),  MODE_TITLE);
    addMenuItem(mapper, group, actions->value(PLAYQUEUE_MODE_ALBUM),  MODE_ALBUM);
    addMenuItem(mapper, group, actions->value(PLAYQUEUE_MODE_ARTIST), MODE_ARTIST);
    addMenuItem(mapper, group, actions->value(PLAYQUEUE_MODE_EXTENDED), MODE_EXTENDED);

    connect(mapper, SIGNAL(mapped(int)), PlaylistWidget::instance(), SLOT(playQueueModeChange(int))); 
    m2->addActions(group->actions());
      
    m_menu->addSeparator();
    m_menu->addAction(actions->value(DIALOG_DB_OPERATION));
    m_menu->addAction(actions->value(ENGINE_AUDIO_EQ));
    m_menu->addSeparator();

    m_menu->addAction(actions->value(DIALOG_PLAYLIST_EDITOR));
    
    m_menu->addSeparator();
    m_menu->addAction(actions->value(APP_MODE_COMPACT));

    m_menu->addSeparator();
    m_menu->addAction(actions->value(APP_SHOW_YAROCK_ABOUT));
    m_menu->addAction(actions->value(APP_SHOW_QT_ABOUT));
    
    m_menu->addSeparator();
    m_menu->addAction(actions->value(APP_QUIT));
  
    setContextMenuPolicy( Qt::CustomContextMenu );
    connect( this, SIGNAL( clicked ( bool ) ), this, SLOT( slot_showMenu() ) );
}


void MainToolButton::slot_showMenu()
{
    //Debug::debug() << "-- MainToolButton --> showMenu";
    updateMultiDbMenu();
    
    QPoint pos( 0, height() );
    m_menu->exec( mapToGlobal( pos ) );
}


void MainToolButton::addMenuItem(QSignalMapper* mapper, QActionGroup* group,
                                 QAction* action, Q_MODE mode) 
{
    group->addAction(action);
    mapper->setMapping(action, mode);
    connect(action, SIGNAL(triggered()), mapper, SLOT(map()));

    // restore action state with current mode
    if (mode == SETTINGS()->_playqueueMode)
        action->setChecked(true);
}

void MainToolButton::updateMultiDbMenu()
{
    //Debug::debug() << "-- MainToolButton --> updateMultiDbMenu";
    if( !DatabaseManager::instance()->multiDb )
    {
        if(m_menuChooseDbAction != 0)
          m_menu->removeAction ( m_menuChooseDbAction );
        
        m_menuChooseDbAction = 0;
        return;
    }
    
    //! Multi Db activated --> show choose database menu
    QActionGroup* group = new QActionGroup(this);
      
    foreach (const QString& name, DatabaseManager::instance()->DB_NAME_LIST())
    {
        QAction *a = new QAction(QIcon(), name, this);
        a->setCheckable(true);

        group->addAction(a);
  
        if(name == DatabaseManager::instance()->DB_NAME)
          a->setChecked(true);

        connect(a, SIGNAL(triggered()), this, SLOT(slot_dbNameClicked()));
    }
    
    m_menuChooseDb->clear();
    m_menuChooseDb->addActions(group->actions());
    
    if(!m_menuChooseDbAction)
       m_menuChooseDbAction = m_menu->insertMenu ( ACTIONS()->value(DIALOG_DB_OPERATION), m_menuChooseDb );

    if( ThreadManager::instance()->isDbRunning()) {
       m_menuChooseDb->setEnabled(false);
    }
    else {
       m_menuChooseDb->setEnabled(true);
    }      
}


void MainToolButton::slot_dbNameClicked()
{
    QAction *action = qobject_cast<QAction *>(sender());
 
    DatabaseManager::instance()->DB_NAME = action->text();   
    //Debug::debug() << "-- MainToolButton --> dbNameClicked = " << action->text();

    emit dbNameChanged();
}






/*
********************************************************************************
*                                                                              *
*    Class CentralToolBar                                                      *
*                                                                              *
********************************************************************************
*/

CentralToolBar::CentralToolBar(QWidget *parent) : QToolBar( "CentralToolBar", parent )
{
    INSTANCE   = this;
  
    this->setObjectName(QString::fromUtf8("CentralToolBar"));
    this->setSizePolicy(QSizePolicy::Minimum, QSizePolicy::Minimum );
    this->setMaximumHeight(28);
    this->setFloatable(false);
    this->setMovable(false);
    this->setContentsMargins( 0, 0, 0, 0);
    this->setToolButtonStyle( Qt::ToolButtonIconOnly );
    this->setIconSize( QSize( 20, 20) );
  
    m_history_current_idx = 0;
  
    QMap<ENUM_ACTION, QAction*> *actions = ACTIONS();
  
    //! NavigatorBar
    m_navigatorBar = new NavigatorBar();
    m_navigatorBar->setModel( MenuModel::instance() );

    //! Prev/Next
    actions->insert(BROWSER_PREV, new QAction(QIcon::fromTheme("go-previous", QIcon(":/images/go-previous.png")),tr("Go back"),this));     
    actions->insert(BROWSER_NEXT, new QAction(QIcon::fromTheme("go-next", QIcon(":/images/go-next.png")),tr("Go forward"),this));     
        
    //! Search field widget
    ui_line_edit = new LineEditClear(this);
    ui_line_edit->setInactiveText(tr("Search in Library"));

    //! Tools Button and Menu
    MainToolButton* main_tb = new MainToolButton(this);

    //! Tools Button 
    ui_goto_button = new QToolButton(this);
    ui_goto_button->setIconSize( QSize( 20, 20 ) );
    ui_goto_button->setIcon(QIcon(":/images/goto-32x32.png")); 
    ui_goto_button->setSizePolicy(QSizePolicy::Minimum, QSizePolicy::Minimum);    
    ui_goto_button->setToolTip(tr("Jump to now playing track")); 
    ui_goto_button->setEnabled(false); 
    ui_goto_button->setPopupMode(QToolButton::InstantPopup);
    ui_goto_button->setArrowType(Qt::NoArrow);
    
    QMenu * jump_menu = new QMenu();
    jump_menu->addAction(actions->value(BROWSER_JUMP_TO_ARTIST));
    jump_menu->addAction(actions->value(BROWSER_JUMP_TO_ALBUM));
    jump_menu->addAction(actions->value(BROWSER_JUMP_TO_TRACK));
    ui_goto_button->setMenu( jump_menu );

    //!Tool Bar Settup
    QWidget *spacer_widget_1 = new QWidget(this);
    spacer_widget_1->setSizePolicy(QSizePolicy::MinimumExpanding, QSizePolicy::Minimum);
    
    QWidget *spacer_widget_2 = new QWidget(this);
    spacer_widget_2->setSizePolicy(QSizePolicy::MinimumExpanding, QSizePolicy::Minimum);
    
   
    this->addWidget( new FixedSpacer( this, QSize(5, 0)) );
    this->addWidget( main_tb );
    this->addAction( actions->value(BROWSER_PREV) );
    this->addAction( actions->value(BROWSER_NEXT) );
    this->addWidget( spacer_widget_1 );
    this->addWidget( m_navigatorBar );    
    this->addWidget( spacer_widget_2 );    
    this->addWidget( ui_line_edit );
    this->addWidget( ui_goto_button );
    this->addWidget( new FixedSpacer( this, QSize(5, 0)) );
    
    //! Signals
    connect(actions->value(BROWSER_PREV), SIGNAL(triggered()), this, SLOT(slot_histo_prev_activated()));
    connect(actions->value(BROWSER_NEXT), SIGNAL(triggered()), this, SLOT(slot_histo_next_activated()));
    connect(ui_line_edit, SIGNAL(textfield_entered()), this, SLOT(slot_send_search()));
    connect(main_tb, SIGNAL(dbNameChanged()), this, SIGNAL(dbNameChanged()));
}


//! PUBLIC

/*******************************************************************************
    CentralToolBar::updateMode
*******************************************************************************/
void CentralToolBar::updateMode(MAIN::ViewMode mode)
{
    ui_line_edit->setEnable(mode != MAIN::ViewContext);  
}

/*******************************************************************************
    CentralToolBar::updateCurrentIndex
      -> called by CentralWidget on menu activation
*******************************************************************************/
void CentralToolBar::updateCurrentIndex(const QModelIndex &idx)
{
    //Debug::debug() << "-- CentralToolBar -> updateCurrentIndex  #1"; 
    if(!idx.isValid())  return;

    //! nouveau dclenchement --> on supprime les next
    for(int i = 0; i < m_history_current_idx; i++)
      m_history_params.removeAt(i);
    
    HistoryParam param = HistoryParam(idx, ui_line_edit->text());
    m_history_params.prepend(param);
    m_history_current_idx = 0;

    ACTIONS()->value(BROWSER_PREV)->setEnabled(m_history_current_idx < m_history_params.size() -1);
    ACTIONS()->value(BROWSER_NEXT)->setEnabled(m_history_current_idx > 0);
  
    //! limite de la taille de l'historique de navigation
    if(m_history_params.size() >= 20 && m_history_current_idx != m_history_params.size() -1)
      m_history_params.takeLast();
}


//! PRIVATE SLOTS
/*******************************************************************************
    CentralToolBar::slot_send_search
*******************************************************************************/
void CentralToolBar::slot_send_search() 
{
    // register the new activation in history
    updateCurrentIndex( m_history_params.at(m_history_current_idx).idx);

    // send search 
    emit searchActivated(ui_line_edit->text());
}

/*******************************************************************************
    CentralToolBar::slot_histo_prev_activated
*******************************************************************************/
void CentralToolBar::slot_histo_prev_activated()
{
    if(m_history_current_idx < m_history_params.size() -1) 
    {
      m_history_current_idx++;

      HistoryParam param = m_history_params.at(m_history_current_idx);

      emit prevNextActivated(param.idx, param.search_pattern);
      ui_line_edit->setText( param.search_pattern );
      ui_line_edit->clearFocus();

      ACTIONS()->value(BROWSER_PREV)->setEnabled(m_history_current_idx < m_history_params.size() -1);
      ACTIONS()->value(BROWSER_NEXT)->setEnabled(m_history_current_idx > 0);
    }
}

/*******************************************************************************
    CentralToolBar::slot_histo_next_activated
*******************************************************************************/
void CentralToolBar::slot_histo_next_activated()
{
    if(m_history_current_idx > 0) 
    {
      m_history_current_idx--;

      HistoryParam param = m_history_params.at(m_history_current_idx);
            
      emit prevNextActivated(param.idx, param.search_pattern);
      ui_line_edit->setText( param.search_pattern );
      ui_line_edit->clearFocus();
      
      ACTIONS()->value(BROWSER_PREV)->setEnabled(m_history_current_idx < m_history_params.size() -1);
      ACTIONS()->value(BROWSER_NEXT)->setEnabled(m_history_current_idx > 0);
    }
}


/*******************************************************************************
    CentralToolBar::set_enable_jump_to
*******************************************************************************/
void CentralToolBar::set_enable_jump_to(bool b)
{
    ui_goto_button->setEnabled(b);   
}
   