/* YaRock Player
* Copyright (C) 2010-2013 Sebastien amardeilh (sebastien.amardeilh+yarock@gmail.com)
*
* This program is free software: you can redistribute it and/or modify
* it under the terms of the GNU General Public License as published by
* the Free Software Foundation, either version 3 of the License, or
* (at your option) any later version.
*
* This program is distributed in the hope that it will be useful,
* but WITHOUT ANY WARRANTY; without even the implied warranty of
* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
* GNU General Public License for more details.
*
* You should have received a copy of the GNU General Public License
* along with this program.  If not, see <http://www.gnu.org/licenses/>.
*/

#ifndef _GRAPHICS_ITEM_H_
#define _GRAPHICS_ITEM_H_

#include <QtGui>

#include "MediaItem/mediaitem.h"

namespace GraphicsItem {
  enum GraphicsType {
      AlbumType             = QGraphicsItem::UserType + 1,
      ArtistType            = QGraphicsItem::UserType + 2,
      AlbumGenreType        = QGraphicsItem::UserType + 3,
      TrackType             = QGraphicsItem::UserType + 4,
      PlaylistType          = QGraphicsItem::UserType + 5,
      StreamType            = QGraphicsItem::UserType + 6
  };
}

/*
********************************************************************************
*                                                                              *
*    Class InfoGraphicItem                                                     *
*                                                                              *
********************************************************************************
*/   
class InfoGraphicItem : public QGraphicsItem
{
  public:
    InfoGraphicItem();
    QString     _text;
    int         _width;

  protected:
    QRectF boundingRect() const;
    void paint(QPainter *p, const QStyleOptionGraphicsItem *, QWidget *);
};

/*
********************************************************************************
*                                                                              *
*    Class CategorieGraphicItem                                                *
*                                                                              *
********************************************************************************
*/   
class CategorieGraphicItem : public QGraphicsItem
{
  public:
    CategorieGraphicItem();
    QString     _catName;
    int         _width;
    
  protected:
    QRectF boundingRect() const;
    void paint(QPainter *p, const QStyleOptionGraphicsItem *, QWidget *);
};


/*
********************************************************************************
*                                                                              *
*    Class AlbumGraphicItem                                                    *
*                                                                              *
********************************************************************************
*/  
class AlbumGraphicItem : public QGraphicsItem
{
  public:
    AlbumGraphicItem();
    MediaItemPtr             media;
    
    int type() const { return GraphicsItem::AlbumType; }

  protected:
    // Implement QGraphicsItem method
    QRectF boundingRect() const;
    void paint(QPainter *p, const QStyleOptionGraphicsItem *, QWidget *);

    void hoverEnterEvent(QGraphicsSceneHoverEvent *event);
    void hoverLeaveEvent(QGraphicsSceneHoverEvent *event);
    void mousePressEvent(QGraphicsSceneMouseEvent *event);
    void mouseReleaseEvent(QGraphicsSceneMouseEvent *e);    
    void mouseDoubleClickEvent ( QGraphicsSceneMouseEvent * event );
    void mouseMoveEvent(QGraphicsSceneMouseEvent *event);
    void contextMenuEvent(QGraphicsSceneContextMenuEvent *event);
  public:
     QStyleOptionViewItemV4 opt;    
};

/*
********************************************************************************
*                                                                              *
*    Class AlbumGraphicItem_v2                                                 *
*                                                                              *
********************************************************************************
*/  
class AlbumGraphicItem_v2 : public AlbumGraphicItem
{
  public:
    AlbumGraphicItem_v2();
    
  protected:
    // Implement QGraphicsItem method
    void paint(QPainter *p, const QStyleOptionGraphicsItem *, QWidget *);
};


/*
********************************************************************************
*                                                                              *
*    Class AlbumGenreGraphicItem                                               *
*                                                                              *
********************************************************************************
*/  
class AlbumGenreGraphicItem : public AlbumGraphicItem
{
  public:
    AlbumGenreGraphicItem();
    QString                 _genre;
    int type() const { return GraphicsItem::AlbumGenreType; }
    
  protected:
    // Implement QGraphicsItem method
    void mouseMoveEvent(QGraphicsSceneMouseEvent *event);
};

/*
********************************************************************************
*                                                                              *
*    Class AlbumGraphicItem_v3                                                 *
*         -> (with playcount)                                                  *
********************************************************************************
*/  
class AlbumGraphicItem_v3 : public AlbumGraphicItem
{
  public:
    AlbumGraphicItem_v3();
    int         _maxPlaycount;
        
  protected:
    // Implement QGraphicsItem method
    QRectF boundingRect() const;
    void paint(QPainter *p, const QStyleOptionGraphicsItem *, QWidget *);
};

/*
********************************************************************************
*                                                                              *
*    Class AlbumGraphicItem_v4                                                 *
*         -> (with rating)                                                     *
********************************************************************************
*/  

class AlbumGraphicItem_v4 : public AlbumGraphicItem
{
  public:
    AlbumGraphicItem_v4();
       
  protected:
    // Implement QGraphicsItem method
    QRectF boundingRect() const;
    void paint(QPainter *p, const QStyleOptionGraphicsItem *, QWidget *);
};

/*
********************************************************************************
*                                                                              *
*    Class ArtistGraphicItem                                                   *
*                                                                              *
********************************************************************************
*/  
class ArtistGraphicItem : public QGraphicsItem
{
  public:
    ArtistGraphicItem();
    MediaItemPtr             media;
    int type() const { return GraphicsItem::ArtistType; }
    
  protected:
    // Implement QGraphicsItem method
    QRectF boundingRect() const;
    void paint(QPainter *p, const QStyleOptionGraphicsItem *, QWidget *);

    void hoverEnterEvent(QGraphicsSceneHoverEvent *event);
    void hoverLeaveEvent(QGraphicsSceneHoverEvent *event);
    void mousePressEvent(QGraphicsSceneMouseEvent *event);
    void mouseReleaseEvent(QGraphicsSceneMouseEvent *e);    
    void mouseDoubleClickEvent ( QGraphicsSceneMouseEvent * event );
    void contextMenuEvent(QGraphicsSceneContextMenuEvent *event);
    void mouseMoveEvent(QGraphicsSceneMouseEvent *event);
  public:
     QStyleOptionViewItemV4 opt;    
};

/*
********************************************************************************
*                                                                              *
*    Class ArtistGraphicItem_v2                                                *
*         -> (with playcount)                                                  *
********************************************************************************
*/  
class ArtistGraphicItem_v2 : public ArtistGraphicItem
{
  public:
    ArtistGraphicItem_v2();
    int         _maxPlaycount;
           
  protected:
    // Implement QGraphicsItem method
    QRectF boundingRect() const;
    void paint(QPainter *p, const QStyleOptionGraphicsItem *, QWidget *);
};

/*
********************************************************************************
*                                                                              *
*    Class ArtistGraphicItem_v3                                                *
*         -> (with rating)                                                     *
********************************************************************************
*/  
class ArtistGraphicItem_v3 : public ArtistGraphicItem
{
  public:
    ArtistGraphicItem_v3();
       
  protected:
    // Implement QGraphicsItem method
    QRectF boundingRect() const;
    void paint(QPainter *p, const QStyleOptionGraphicsItem *, QWidget *);
};

/*
********************************************************************************
*                                                                              *
*    Class TrackGraphicWidget                                                  *
*      -> used in PLAYLIST VIEW                                                *
*      -> draw only number/track name                                          *
********************************************************************************
*/  
class TrackGraphicItem : public QGraphicsItem
{
  public:
    TrackGraphicItem();
    int                      _width;
    MediaItemPtr             media;
    int type() const { return GraphicsItem::TrackType; }
    
  protected:
    // Implement QGraphicsItem method
    QRectF boundingRect() const;
    void paint(QPainter *p, const QStyleOptionGraphicsItem *, QWidget *);

    void hoverEnterEvent(QGraphicsSceneHoverEvent *event);
    void hoverLeaveEvent(QGraphicsSceneHoverEvent *event);
    void mousePressEvent(QGraphicsSceneMouseEvent *event);
    void mouseReleaseEvent(QGraphicsSceneMouseEvent *e);    
    void mouseDoubleClickEvent ( QGraphicsSceneMouseEvent *event );
    void contextMenuEvent(QGraphicsSceneContextMenuEvent *event);
    void mouseMoveEvent(QGraphicsSceneMouseEvent *event);
  public:
     QStyleOptionViewItemV4 opt;    
};

/*
********************************************************************************
*                                                                              *
*    Class TrackGraphicItem_v2                                                 *
*      -> used in TRACKS VIEW                                                  *
*      -> draw number/track name/album/duration/rating                         *
*                                                                              *
********************************************************************************
*/  
class TrackGraphicItem_v2  : public TrackGraphicItem
{
  public:
    TrackGraphicItem_v2();
    
  protected:
    // Implement QGraphicsItem method
    QRectF boundingRect() const;
    void paint(QPainter *p, const QStyleOptionGraphicsItem *, QWidget *);
};

/*
********************************************************************************
*                                                                              *
*    Class TrackGraphicItem_v3                                                 *
*      -> used in HISTORY VIEW                                                 *
*      -> work for stream or track                                             *
*      -> draw track artist/album/title/rating or stream url name              *
********************************************************************************
*/ 
class TrackGraphicItem_v3 : public TrackGraphicItem
{
  public:
    TrackGraphicItem_v3();
    
  protected:
    QRectF boundingRect() const;
    void paint(QPainter *p, const QStyleOptionGraphicsItem *, QWidget *);
};

/*
********************************************************************************
*                                                                              *
*    Class PlaylistGraphicItem                                                 *
*                                                                              *
********************************************************************************
*/ 
class PlaylistGraphicItem : public QGraphicsItem
{
  public:
    PlaylistGraphicItem();
    MediaItemPtr             media;
    int type() const { return GraphicsItem::PlaylistType; }
        
  protected:
    // Implement QGraphicsItem method
    QRectF boundingRect() const;
    void paint(QPainter *p, const QStyleOptionGraphicsItem *, QWidget *);

    void hoverEnterEvent(QGraphicsSceneHoverEvent *event);
    void hoverLeaveEvent(QGraphicsSceneHoverEvent *event);
    void mousePressEvent(QGraphicsSceneMouseEvent *event);
    void mouseReleaseEvent(QGraphicsSceneMouseEvent *e);    
    void mouseDoubleClickEvent ( QGraphicsSceneMouseEvent * event );
    void mouseMoveEvent(QGraphicsSceneMouseEvent *event);
    void contextMenuEvent(QGraphicsSceneContextMenuEvent *event);
  private:
     QStyleOptionViewItemV4 opt;    
};


/*
********************************************************************************
*                                                                              *
*    Class StreamGraphicItem                                                   *
*                                                                              *
********************************************************************************
*/ 
class StreamGraphicItem : public QGraphicsItem
{
  public:
    StreamGraphicItem();
    int type() const { return GraphicsItem::StreamType; }
    MediaItemPtr        media;
    int                _width;

  protected:
    // Implement QGraphicsItem method
    QRectF boundingRect() const;
    void paint(QPainter *p, const QStyleOptionGraphicsItem *, QWidget *);

    void hoverEnterEvent(QGraphicsSceneHoverEvent *e);
    void hoverLeaveEvent(QGraphicsSceneHoverEvent *e);
    void mousePressEvent(QGraphicsSceneMouseEvent *e);
    void mouseReleaseEvent(QGraphicsSceneMouseEvent *e);    
    void mouseDoubleClickEvent ( QGraphicsSceneMouseEvent * event );
    void mouseMoveEvent(QGraphicsSceneMouseEvent *event);
    void contextMenuEvent(QGraphicsSceneContextMenuEvent *event);
  private:
     QStyleOptionViewItemV4 opt;
};


#endif // _GRAPHICS_ITEM_H_
