/* YaRock Player
* Copyright (C) 2010-2013 Sebastien amardeilh (sebastien.amardeilh+yarock@gmail.com)
*
* This program is free software: you can redistribute it and/or modify
* it under the terms of the GNU General Public License as published by
* the Free Software Foundation, either version 3 of the License, or
* (at your option) any later version.
*
* This program is distributed in the hope that it will be useful,
* but WITHOUT ANY WARRANTY; without even the implied warranty of
* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
* GNU General Public License for more details.
*
* You should have received a copy of the GNU General Public License
* along with this program.  If not, see <http://www.gnu.org/licenses/>.
*/

#ifndef _PLAYLIST_MODEL_H_
#define _PLAYLIST_MODEL_H_

#include <QtGui>
#include <QMutex>

#include "MediaItem/mediaitem.h"
#include "MediaItem/mediamimedata.h"
#include "threadmanager.h"

/*
********************************************************************************
*                                                                              *
*    Class PlaylistModel                                                       *
*                                                                              *
********************************************************************************
*/
class PlaylistModel : public QAbstractListModel 
{
Q_OBJECT
    static PlaylistModel* INSTANCE;  

  public:
    PlaylistModel(QWidget *parent);
    
    //! return global instance
    static PlaylistModel* instance() { return INSTANCE; }    
    
    void setThreadManager(ThreadManager* tm) {m_thread_manager = tm;}
    
    //! inherited from QAbstractListModel
    int rowCount(const QModelIndex &parent = QModelIndex()) const;
    QVariant data(const QModelIndex &index, int role) const;
    bool removeRows(int position, int count, const QModelIndex &parent);

    //! drag and drop
    Qt::ItemFlags flags(const QModelIndex &index) const;
    QStringList mimeTypes() const;
    Qt::DropActions supportedDropActions() const;
    QMimeData* mimeData( const QModelIndexList &indexes ) const;
    bool dropMimeData(const QMimeData *data,
                      Qt::DropAction action, int row, int column,
                      const QModelIndex &parent);

    void move(const MediaMimeData *data, int row);
    
    //! custom methods
    bool rowExists( int row ) const { return (( row >= 0 ) && ( row < _listMediaItem.size() ) ); }
    int rowForMediaItem(const MediaItemPtr mi);

    MediaItemPtr MediaItemAt( int row ) const;
    QString getListDuration();
    
    //! get Unplayed mediaItem
    QList<MediaItemPtr> unplayedMediaItem();
    
    //! broken MediaItem method
    void updateMediaItemBroken(const QString&);
    void updateMediaItemPlaying(const QString&);
    
    //! Requested MediaItem method
    void setRequestedMediaItem(int row);
    MediaItemPtr getRequestedMediaItem() const;
    MediaItemPtr getPlayingMediaItem() const;
    int getPlayingRow();
    void stopPlayingMediaItem();
    
    //! add/insert mediaitem  
    void addMediaItem(const MediaItemPtr mediaitem);
    void addMediaItems(QList<MediaItemPtr>);
    void insertMediaItem(const MediaItemPtr mediaitem, int pos=-1);
    void removeDuplicate();    
      
    //! Stop After media Action
    MediaItemPtr get_stop_after_media() const;
    void set_stop_after_media(MediaItemPtr media);
      
  public slots:
    void clear();
    void skipBackward();
    void skipForward();
    void needUpdate();
  
  signals:
    void needSelectionAfterMove(QList<MediaItemPtr>);

  private:
    ThreadManager       *m_thread_manager;
    MediaItemPtr         _playingMediaItem;
    MediaItemPtr         _requestedMediaItem;
    MediaItemPtr         m_stop_after_media;
    QList<MediaItemPtr>  _listMediaItem;
    QMutex               _mutex;
};

#endif // _PLAYLIST_MODEL_H_
