/* YaRock Player
* Copyright (C) 2010-2013 Sebastien amardeilh (sebastien.amardeilh+yarock@gmail.com)
*
* This program is free software: you can redistribute it and/or modify
* it under the terms of the GNU General Public License as published by
* the Free Software Foundation, either version 3 of the License, or
* (at your option) any later version.
*
* This program is distributed in the hope that it will be useful,
* but WITHOUT ANY WARRANTY; without even the implied warranty of
* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
* GNU General Public License for more details.
*
* You should have received a copy of the GNU General Public License
* along with this program.  If not, see <http://www.gnu.org/licenses/>.
*/


#ifndef _STREAMMODEL_H_
#define _STREAMMODEL_H_

#include "MediaItem/mediaitem.h"

#include <QList>
#include <QObject>
#include <QString>

/*
********************************************************************************
*                                                                              *
*    Class StreamModel                                                         *
*                                                                              *
********************************************************************************
*/
class StreamModel  : public QObject
{
Q_OBJECT
    static StreamModel* INSTANCE;

  public:
    StreamModel(QObject *parent);
    
    //! return global instance
    static StreamModel* instance() { return INSTANCE; }    
     
    //! liste streal update
    void updateListeStream(const QList<MediaItemPtr> &liste);

    int itemCount() {return m_listStream.size();}

    //! get Stream Item method
    MediaItemPtr getStreamItem(int row);

    //! playing status method
    void     stopPlayingItem();
    void     updatePlayingItem(const QString& urlName);

    //! broken status method
    void     updateBrokenItem(const QString& urlName);
    
    //! Favorites status method
    bool itemIsFavorite(const MediaItemPtr item);
    void setItemIsFavorite(const MediaItemPtr item, bool isfavorite);
     
    //! filtering method
    void setFilter(const QString & f) {m_filter_pattern = f;}
    bool isStreamFiltered(const int row);
    
    void clear();

  signals:
    void signalPlayingStatusChanged();

  private:
    QList<MediaItemPtr>   m_listStream;
    QString               m_filter_pattern;
};

#endif // _STREAMMODEL_H_ 
