/* YaRock Player
* Copyright (C) 2010-2013 Sebastien amardeilh (sebastien.amardeilh+yarock@gmail.com)
*
* This program is free software: you can redistribute it and/or modify
* it under the terms of the GNU General Public License as published by
* the Free Software Foundation, either version 3 of the License, or
* (at your option) any later version.
*
* This program is distributed in the hope that it will be useful,
* but WITHOUT ANY WARRANTY; without even the implied warranty of
* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
* GNU General Public License for more details.
*
* You should have received a copy of the GNU General Public License
* along with this program.  If not, see <http://www.gnu.org/licenses/>.
*/

#include "seekslider.h"
#include "MediaItem/mediaitem.h"
#include "EnginePlayer/engineplayer.h"
#include "debug.h"

#include <QtGui/QApplication>

/*
********************************************************************************
*                                                                              *
*    Class SeekSlider                                                          *
*                                                                              *
********************************************************************************
*/   

SeekSlider::SeekSlider(Phonon::MediaObject *mo, QWidget *parent) : QProgressBar(parent)
{
    //! build seekslider
    this->setOrientation(Qt::Horizontal);
    this->setEnabled(false);
    this->setTextVisible(false);
    this->setSizePolicy(QSizePolicy::Expanding, QSizePolicy::Expanding);
    this->setMaximumHeight(6);
    this->setMinimumHeight(6);
    
    media   = mo;
    
    //! signals
    connect(media, SIGNAL(stateChanged(Phonon::State, Phonon::State)),this, SLOT(slot_stateChanged(Phonon::State, Phonon::State)));

    // change because of bad phonon emittion for totalTimeChanged (before current source change)
    //connect(media, SIGNAL(totalTimeChanged(qint64)), this, SLOT(_length(qint64)));
    connect(EnginePlayer::instance(), SIGNAL(trackTotalTimeChanged(qint64)), this, SLOT(slot_length(qint64)));
    
    connect(media, SIGNAL(tick(qint64)), this, SLOT(slot_tick(qint64)));
    connect(media, SIGNAL(seekableChanged(bool)), this, SLOT(slot_seekableChanged(bool)));
    connect(media, SIGNAL(currentSourceChanged(const Phonon::MediaSource&)), this, SLOT(slot_currentSourceChanged()));
    
    this->setEnabled(true);
    reset ();
    setCursor(Qt::PointingHandCursor);

}

void SeekSlider::_seek(int msec)
{
    if (!ticking && media) {
        media->seek(msec);
    }
}

void SeekSlider::mousePressEvent ( QMouseEvent * event ) 
{
    _seek((double)event->x()*maximum()/width());
}

void SeekSlider::slot_tick(qint64 msec)
{
    //if(enable) {
      //Debug::debug() << "-- SeekSlider -> slot_tick(qint64 msec) msec = " << msec;
      ticking = true;
      this->setValue(msec);
      ticking = false;
    //}
}

void SeekSlider::slot_length(qint64 msec)
{
    //Debug::debug() << "-- SeekSlider -> slot_length(qint64 msec) msec = " << msec;
  //if(enable) {
    ticking = true;
    if(msec <= 0) 
    {
      this->setRange(0, 1);
      this->setValue(0);  
    }
    else 
    {
      this->setRange(0, msec);
    }
    ticking = false;
  //}     
}

void SeekSlider::slot_seekableChanged(bool isSeekable)
{
    //Debug::debug() << "-- SeekSlider -> slot_seekableChanged  isSeekable" << isSeekable;
    //! FIXME I don't understand why it's not working with Gstreamer !!!
    //! so return and do nothing 
    return; 
    
    
    if (!media || ! isSeekable) {
        //_stop();
        this->setRange(0, 1);
        this->setValue(0);      
        return;
    }
    switch (media->state()) {
    case Phonon::PlayingState:
        if (media->tickInterval() == 0) {
            // if the tick signal is not enabled the slider is useless
            // set the tickInterval to some common value
            //Debug::debug() << "-- SeekSlider -> _stateChanged   setTickInterval";
            media->setTickInterval(1000);
        }
    case Phonon::BufferingState:
    case Phonon::PausedState:
        //this->setEnabled(true);
        enable = true;
        break;
    case Phonon::StoppedState:
    case Phonon::LoadingState:
    case Phonon::ErrorState:
        //_stop();
        this->setRange(0, 1);
        this->setValue(0);          
        break;
    }
}

void SeekSlider::slot_currentSourceChanged()
{
    //this releases the mouse and makes the seek slider stop seeking if the current source has changed
    QMouseEvent event(QEvent::MouseButtonRelease, QPoint(), Qt::LeftButton, 0, 0);
    QApplication::sendEvent(this, &event);
}

void SeekSlider::slot_stateChanged(Phonon::State newState, Phonon::State /* oldState */)
{
    //Debug::debug() << "-- SeekSlider -> slot_stateChanged newState" << newState;
    //! FIXME I don't understand why it's not working with Gstreamer !!!
    //! so return and do nothing 
    if(newState == Phonon::StoppedState) {
        this->setValue(0);        
    }
    return; 

    if (!media || !media->isSeekable()) {
        _stop();
        return;
    }
    switch (newState) {
    case Phonon::PlayingState:
        if (media->tickInterval() == 0) {
            // if the tick signal is not enabled the slider is useless
            // set the tickInterval to some common value
            //Debug::debug() << "-- SeekSlider -> slot_stateChanged   setTickInterval";
            media->setTickInterval(1000);
        }
    case Phonon::BufferingState:
    case Phonon::PausedState:
        //this->setEnabled(true);
        enable = true;
        break;
    case Phonon::StoppedState:
    case Phonon::LoadingState:
    case Phonon::ErrorState:
        //_stop();
        this->setRange(0, 1);
        this->setValue(0);  
        break;
    }
}


void SeekSlider::_stop() 
{
    //Debug::debug() << " ### SeekSlider -> STOP";  
    //this->setEnabled(false);	
    enable = false;
    this->setRange(0, 1);
    this->setValue(0);  
}
