/* YaRock Player
* Copyright (C) 2010-2013 Sebastien amardeilh (sebastien.amardeilh+yarock@gmail.com)
*
* This program is free software: you can redistribute it and/or modify
* it under the terms of the GNU General Public License as published by
* the Free Software Foundation, either version 3 of the License, or
* (at your option) any later version.
*
* This program is distributed in the hope that it will be useful,
* but WITHOUT ANY WARRANTY; without even the implied warranty of
* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
* GNU General Public License for more details.
*
* You should have received a copy of the GNU General Public License
* along with this program.  If not, see <http://www.gnu.org/licenses/>.
*/

#include "globalshortcuts.h"
#include "yarocksettings.h"

#include "EnginePlayer/engineplayer.h"

#include <QtDebug>

/*
********************************************************************************
*                                                                              *
*    Class GlobalShortcuts                                                     *
*                                                                              *
********************************************************************************
*/
GlobalShortcuts::GlobalShortcuts(QMainWindow *mw) : QObject(mw)
{
    //! init
   _shortcuts["play"]        = new QxtGlobalShortcut(this);
   //_shortcuts["pause"] = new QxtGlobalShortcut(this);
   _shortcuts["stop"]        = new QxtGlobalShortcut(this);
   _shortcuts["next_track"]  = new QxtGlobalShortcut(this);         //Next track
   _shortcuts["prev_track"]  = new QxtGlobalShortcut(this);         //Previous track
   _shortcuts["inc_volume"]  = new QxtGlobalShortcut(this);         //Increase volume
   _shortcuts["dec_volume"]  = new QxtGlobalShortcut(this);         //Decrease volume
   _shortcuts["mute_volume"] = new QxtGlobalShortcut(this);         //mute/unmute volume
   
   //_shortcuts["show_hide"] = new QxtGlobalShortcut(this);          //Show/Hide
   //_shortcuts["shuffle_mode"] = new QxtGlobalShortcut(this);       //Change shuffle mode
   //_shortcuts["repeat_mode"] = new QxtGlobalShortcut(this);        //Change repeat mode
   
   //!connection 
   connect(_shortcuts["play"], SIGNAL(activated()),mw, SLOT(playOrPause()));
   connect(_shortcuts["stop"], SIGNAL(activated()),mw, SLOT(stopPlayer()));
   connect(_shortcuts["next_track"], SIGNAL(activated()),mw, SLOT(playNext()));
   connect(_shortcuts["prev_track"], SIGNAL(activated()),mw, SLOT(playPrev()));
   connect(_shortcuts["inc_volume"], SIGNAL(activated()),this, SLOT(incVolume()));
   connect(_shortcuts["dec_volume"], SIGNAL(activated()),this, SLOT(decVolume()));
   connect(_shortcuts["mute_volume"], SIGNAL(activated()),this, SLOT(muteVolume()));

   reloadSettings();
}

void GlobalShortcuts::reloadSettings() 
{
   _shortcuts["play"]->setShortcut(QKeySequence::fromString(SETTINGS()->_shortcutsKey["play"]));
   _shortcuts["stop"]->setShortcut(QKeySequence::fromString(SETTINGS()->_shortcutsKey["stop"]));
   _shortcuts["prev_track"]->setShortcut(QKeySequence::fromString(SETTINGS()->_shortcutsKey["prev_track"]));
   _shortcuts["next_track"]->setShortcut(QKeySequence::fromString(SETTINGS()->_shortcutsKey["next_track"]));
   _shortcuts["inc_volume"]->setShortcut(QKeySequence::fromString(SETTINGS()->_shortcutsKey["inc_volume"]));
   _shortcuts["dec_volume"]->setShortcut(QKeySequence::fromString(SETTINGS()->_shortcutsKey["dec_volume"]));
   _shortcuts["mute_volume"]->setShortcut(QKeySequence::fromString(SETTINGS()->_shortcutsKey["mute_volume"]));
}


void GlobalShortcuts::incVolume()
{      
    int percent = EnginePlayer::instance()->volume() < 100 ? EnginePlayer::instance()->volume() + 1 : 100;
    EnginePlayer::instance()->setVolume(percent);
}

void GlobalShortcuts::decVolume()
{
    int percent = EnginePlayer::instance()->volume() > 0 ? EnginePlayer::instance()->volume() -1 : 0;
    EnginePlayer::instance()->setVolume(percent);  
}

void GlobalShortcuts::muteVolume()
{
    EnginePlayer::instance()->setMuted( !EnginePlayer::instance()->isMuted() );
}

