/* YaRock Player
* Copyright (C) 2010-2013 Sebastien amardeilh (sebastien.amardeilh+yarock@gmail.com)
*
* This program is free software: you can redistribute it and/or modify
* it under the terms of the GNU General Public License as published by
* the Free Software Foundation, either version 3 of the License, or
* (at your option) any later version.
*
* This program is distributed in the hope that it will be useful,
* but WITHOUT ANY WARRANTY; without even the implied warranty of
* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
* GNU General Public License for more details.
*
* You should have received a copy of the GNU General Public License
* along with this program.  If not, see <http://www.gnu.org/licenses/>.
*/
#ifndef _THREAD_MANAGER_H_
#define _THREAD_MANAGER_H_

#include "mediaitem.h"


#include <QObject>
#include <QThreadPool>
#include <QList>
#include <QUrl>
#include <QString>
#include <QStringList>
#include <QMap>

// thread class
class DataBaseBuilder;      // thread for building database for music files
class CollectionPopulator;  // thread to populate CollectionModel 
class PlsFilePopulator;     // thread to populate PlsFileModel
class CoverThread;          // thread to search cover art from lastfm

// qrunnbale class
class PlaylistPopulator;    // thread to populate playlist
class PlaylistWriter;       // thread to save playlist to file
class PlaylistDbWriter;     // thread to save playlist to Database

class PlaylistModel;

enum E_MODEL_TYPE {
                   MODEL_COLLECTION = 0, 
                   MODEL_PLAYLIST   = 1, 
                   MODEL_ALL        = 2
                   };
                   
/*
********************************************************************************
*                                                                              *
*    Class ThreadManager                                                       *
*                                                                              *
********************************************************************************
*/                   
class ThreadManager : public QObject
{
Q_OBJECT
  public:
    explicit ThreadManager();
    static ThreadManager    *INSTANCE;
    static ThreadManager* instance() { return INSTANCE; }
    
    ~ThreadManager();
    
    void stopThread();
    
    // playlist model
    void setModel(PlaylistModel* model);

    
    // Playlist Populator Thread
    void playlistAddFiles(const QStringList &files);
    void playlistAddFile(const QString &file);
    void playlistAddMediaItems(QList<MediaItemPtr> list, int playlist_row=-1);
    void playlistAddUrls(QList<QUrl> listUrl, int playlist_row=-1);

    // Playlist Writer Thread
    void playlistSaveToFile(const QString &filename);
    void playlistSaveToDb(const QString &name, int db_id=-1);
    
    // Database Builder Thread
    void databaseBuild(QStringList listDir);
    bool isDbRunning();
    
    // Model Populator Thread
    void populateCollectionModel();
    void populatePlsFileModel();
  
    // Cover Search thread 
    void startCoverSearch();
    void startCoverSearch(const QString& artist, const QString& album);
  
  private:
    enum E_THREAD {DB_THREAD, POPULATOR_C_THREAD, POPULATOR_P_THREAD, COVER_SEARCH_THREAD};
    void cancelThread(E_THREAD thread);
  
  private slots:
    void slot_load_async(QString,int);
    void dbBuildProgressChanged(int progress);
    void dbBuildFinish();
    
    void modelCollectionPopulated();
    void modelCollectionPopulatingChanged(int progress);
    void modelCollectionPopulatingRunning();
    
    void modelPlaylistPopulated();
    void modelPlaylistPopulatingChanged(int progress);

    void coverSearchFinished();
    void coverSearchProgress(int progress);   
      
  private:     
    QThreadPool            *_threadPool;          // QRunnable manager
    PlaylistPopulator      *_playlistPopulator;   // QRunnable
    PlaylistWriter         *_playlistWriter;      // QRunnable
    PlaylistDbWriter       *_playlistDbWriter;    // QRunnable
    
    DataBaseBuilder        *_databaseBuilder;     // QThread    
    CollectionPopulator    *_collectionPopulator; // QThread
    PlsFilePopulator       *_plsFilePopulator;    // QThread
    CoverThread            *_coverThread;         // QThread

    PlaylistModel          *m_model;
    
    //! messages Id for StatusWidget management
    QMap<QString, uint>    messageIds;
            
  signals:
    void playlistPopulated();
    void playlistSaved();
    void dbBuildFinished();
    void dbBuildProgress(int);
    void modelPopulationFinished(E_MODEL_TYPE type);
};

#endif // _THREAD_MANAGER_H_
